# frozen_string_literal: true

require_relative "ArseniysWeatherApp/version"
require "faraday"
require "geokit"

module ArseniysWeatherApp
  class Error < StandardError; end

  class Requests
    def self.weather_json(location: [59.9573887, 30.3430287],
                          lang: "ru_RU", forecasts: 7,
                          hours: false, extra: false)
      url = "https://api.weather.yandex.ru/v2/forecast"
      params = { "lat" => location[0], "lon" => location[1],
                 "lang" => lang, "limit" => forecasts,
                 "hours" => hours, "extra" => extra }
      header = { "X-Yandex-API-Key" => "01cafac0-8fd8-48ca-956a-f442580428a0" }
      response = Faraday.get(url, params, header)
      JSON.parse(response.body)
    end

    def self.ip_addr
      url = "https://obscure-temple-73986.herokuapp.com/"
      response = Faraday.get(url, { "get_ip" => "yes" })
      JSON.parse(response.body)["ip_addr"]
    end
  end

  class GeoInfo
    def self.spb_location
      { lat: 59.9573887, lon: 30.3430287 }
    end

    def self.location_by_ip
      Geokit::Geocoders::IpstackGeocoder.api_key = "a1b3e41fa2284fb6fd4db50b8629e884"
      geokit = Geokit::Geocoders::IpstackGeocoder.geocode(Requests.ip_addr)
      { lat: geokit.lat, lon: geokit.longitude }
    end
  end

  class Forecast
    def initialize(lat:, lon:)
      @location = [lat, lon]
    end

    def today
      Requests.weather_json(location: @location, forecasts: 1, hours: true)
    end

    def three_days
      Requests.weather_json(location: @location, forecasts: 3, hours: true)
    end
  end

  class Cleaners
    def self.avg_temperature(forecast)
      forecast = forecast_json(forecast)
      avg_temperature = {}
      day_name = ["today", "tomorrow", "day after tomorrow"]
      forecast["forecasts"].each_with_index do |day, i|
        avg_temperature[day_name[i]] = { morning: day["parts"]["morning"]["temp_avg"],
                                         day: day["parts"]["day"]["temp_avg"],
                                         evening: day["parts"]["evening"]["temp_avg"],
                                         night: day["parts"]["night"]["temp_avg"] }
      end
      avg_temperature
    end

    def self.forecast_json(parts)
      parts["forecasts"].each do |day|
        day["parts"].delete_if { |key| %w[day_short night_short].include?(key) }
      end
      parts
    end

    def self.fact(parts)
      parts["fact"]["temp"]
    end
  end

  class LocationWeatherForecast
    def initialize(lat:, lon:)
      @forecast = Forecast.new(lat: lat, lon: lon)
    end

    def fact
      Cleaners.fact(@forecast.today)
    end

    def today_avg_temperature
      Cleaners.avg_temperature(@forecast.today)
    end

    def three_days_avg_temperature
      Cleaners.avg_temperature(@forecast.three_days)
    end

    def today_forecast_json
      Cleaners.forecast_json(@forecast.today)
    end

    def three_days_forecast_json
      Cleaners.forecast_json(@forecast.three_days)
    end
  end

  class MyWeatherForecast
    def initialize
      location = GeoInfo.location_by_ip
      @location_weather = LocationWeatherForecast.new(lat: location[:lat], lon: location[:lon])
    end

    def method_missing(method_name)
      raise NoMethodError unless @location_weather.methods.include?(method_name)

      @location_weather.method(method_name).call
    end

    def respond_to_missing?(method_name, include_private = false)
      @location_weather.methods.include?(method_name)
    end
  end
end
